require 'logger'
require 'rubygems'
require 'yaparc'



module OpenEHR
  module ADL
    module Scanner
      module DADL
        # c.f. http://www.openehr.org/svn/ref_impl_eiffel/TRUNK/components/adl_parser/src/syntax/adl/parser/adl_scanner.l
        @@dadl_reserved = {
          'true' => :SYM_TRUE, #[Tt][Rr][Uu][Ee] -- -> SYM_TRUE 
          'false' => :SYM_FALSE, # [Ff][Aa][Ll][Ss][Ee] -- -> SYM_FALSE 
          'infinity' => :SYM_INFINITY # [Ii][Nn][Ff][Ii][Nn][Ii][Tt][Yy] -- -> SYM_INFINITY 
        }

        LOG = Logger.new('log/scanner.log')

        class RootScanner
          include Yaparc::Parsable
          def initialize
            @parser = lambda do |input|
              Yaparc::Alt.new(
                              V_QUALIFIED_TERM_CODE_REF.new,
                              V_LOCAL_TERM_CODE_REF.new
                              )
            end
          end
        end

        class V_QUALIFIED_TERM_CODE_REF
          include Yaparc::Parsable
          def initialize
            @parser = lambda do |input|
              Yaparc::Apply.new(Yaparc::Regex.new(/\A\[[a-zA-Z0-9()\._-]+::[a-zA-Z0-9\._-]+\]/)) do |match|
                LOG.info("V_QUALIFIED_TERM_CODE_REF: #{match}")
                [:V_QUALIFIED_TERM_CODE_REF, match]
              end
            end
          end
        end

        class V_LOCAL_TERM_CODE_REF
          include Yaparc::Parsable
          def initialize
            @parser = lambda do |input|
              Yaparc::Apply.new(Yaparc::Regex.new(/\A\[[a-zA-Z0-9][a-zA-Z0-9._\-]*\]/)) do |match|
                LOG.info("V_TERM_CODE_REF: #{match}")
                [:V_LOCAL_TERM_CODE_REF, match]
              end
            end
          end
        end
      end # of DADL
      module CADL
        # c.f. http://www.openehr.org/svn/ref_impl_eiffel/TRUNK/components/adl_parser/src/syntax/cadl/parser/cadl_scanner.l
      end
    end
  end
end
